/* File Measure.c */


/* System include files */
extern "C" {
 #include <math.h>
 #include <stdlib.h>
 #include <graphics.h>
 #include <stdio.h>
 #include <string.h>
}

/* Custom include files */
// Note: in later stage no hardware dependent include files to be put in here!!!
// This file serves as a common measurement file!!!
#include "ad_tpex.h"
#include "ad_tp508.h"
#include "const.h"
#include "types.h"
#include "addmath.h"
#include "globvars.h"
#include "measure.h"

/* Software interrupts inits... */
#pragma intrinsic _enable
#pragma intrinsic _disable

#if defined __cplusplus
  #define __CPPARGS ...
#else
  #define __CPPARGS
#endif


// Local High-Level Measurement Functions
void InitializeTP508 (void);
void ResetTP508(void);
void DeterminePrePostSamples (EQUIP_OVERALL_INFO* pOverallInfoLIVE, long* pNumPreSamples, long* pfNumPostSamples);
BOOL IsHardwarePresent (void);
void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies);
void StartMeasuringTP508 (MEASDEP* pMeasDependencies, EQUIP_OVERALL_INFO* pLIVEInfo);

/* Prototypes for interrupt functions */
void interrupt (*oldtimer)(__CPPARGS);
void interrupt newtimer(__CPPARGS);

/* Global variables */
long lTimerCount;



/*
 * Measure.c function definitions...
 *
 * NAME:
 *   BOOL IsHardwarePresent (void)
 * DESCRIPTION:
 *   This routine checks whether the hardware of the TP508 is present
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   returnvalue  TRUE   hardware is     present
 *                FALSE  hardware is not present
 */
BOOL IsHardwarePresent (void)
{
  int nCount, nValue = 0;

  for (nCount =0; nCount < 10; nCount++)
  {
    nValue+=inportb (Ad_PortAddress + 3);
  }
  if (nValue == 0)
  {
    return (TRUE);
  }
  else /* nValue <> 0 */
  {
    return (FALSE);
  }
}  //IsHardwarePresent


/* NAME:
 *   void InitializeTP508 (void)
 * DESCRIPTION:
 *   This routine initializes the timer interrupt,
 *     checks the presence of the TP508 card,
 *     initializes the DA converters of the card
 *     calls the calibration routines
 *   If all is successfull, the program continues, otherwise the program is
 *   terminated with an error code 1.
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void InitializeTP508 (void)
{

 int nChannelIndex, nStand;
 char TextBuffer[80];

/*
 * Replace existing timer-interrupt
 */
 oldtimer = getvect( 0x1c );
 setvect( 0x1c, newtimer);

/*
 * check whether the card is present
 */
 if (!IsHardwarePresent)
 {
 /*
  * No it is not present
  */
  cleardevice();
  sprintf (TextBuffer, "Could not find TP508-Board in your PC!!!");
  outtextxy (50, 220, TextBuffer);
  sprintf (TextBuffer, "Please connect TP508-Board in your PC prior to executing this program...");
  outtextxy (50, 250, TextBuffer);
  closegraph();
  exit (1);
 }

/*
 * clear the waveform memory at the card
 */
 ClearSRAM ();

/*
 * initialize the AD converters
 */
 Offset_AD[0] = -1.0;
 Offset_AD[1] = -1.0;
 for (nChannelIndex = 0; nChannelIndex < NUMCHANNELS; nChannelIndex++)
   for (nStand = 0; nStand < MAXVOLTDIVSTAND; nStand++)
     DA_Value [nChannelIndex] [nStand] = 0;
 for (nChannelIndex = 0; nChannelIndex < NUMCHANNELS; nChannelIndex++)
   for (nStand = 0; nStand < MAXVOLTDIVSTAND; nStand++)
     Ref_Value [nChannelIndex] [nStand] = 127;
 bCanMeasure = TRUE;

/*
 * start calibrating the TP508
 */
 sprintf (TextBuffer, "Calibrating TP508, please wait...");
 outtextxy (50, 220, TextBuffer);
 if (!Calibrate_TP508())
 {
  cleardevice();
  sprintf (TextBuffer, "Calibration has not been succesful!!!");
  outtextxy (50, 220, TextBuffer);
  sprintf (TextBuffer, "Please check your TP508-BOARD...");
  outtextxy (50, 250, TextBuffer);

  closegraph();
  exit (1);
 }
 cleardevice();

}//InitializeTP508



/* NAME:
 *   void interrupt newtimer(__CPPARGS)
 * DESCRIPTION:
 *   This is the new timer interrupt routine
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void interrupt newtimer(__CPPARGS)
{
  oldtimer();
  _disable();
  lTimerCount++;
  _enable ();
} //newtimer(__CPPARGS)



/* NAME:
 *   void ResetTP508 (void)
 * DESCRIPTION:
 *   This routine resets the TP508 after use and resets the
 *   original timer interrupt
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS;
 *   -
 */
void ResetTP508 (void)
{
 outportb (int(Ad_PortAddress + 4), (unsigned char)3);
 outportb (int(Ad_PortAddress + 5), (unsigned char)0);
 // Reset to old timer-interrupt
 setvect(0x1c, oldtimer);
}//ResetTP508



/* NAME:
 *   void StartMeasuringTP508( MEASDEP* pMeasDependencies,
 *                             EQUIP_OVERALL_INFO* pLIVEInfo
 *                           )
 * DESCRIPTION:
 *   This routine performs one measurement with the TP508 card and displays
  *  the measured data like an oscilloscope.
 * INPUT PARAMETERS:
 *   pMeasDependencies  settings of the instrument (v/div, s/div etc.)
 *   pLIVEInfo          measurement mode (freeze or live)
 * OUTPUT PARAMETERS:
 *   -
 */
void StartMeasuringTP508( MEASDEP*            pMeasDependencies,
                          EQUIP_OVERALL_INFO* pLIVEInfo
                        )
{
 TP508_MEASDEP TP508_OverallInfo;
 long int SwitchSettings;

/*
 * prepare the data for the coupling switches
 */
 SwitchSettings = Ch_Switch_Value[0][pMeasDependencies->Switch[CHANNEL1]] +
                  Ch_Switch_Value[1][pMeasDependencies->Switch[CHANNEL2]];

/*
 * Adjust the offsets for ch1 and ch2
 */
 AdjustOffsetDA( CHANNEL1,
                 pMeasDependencies->DCLevel[0] +
                 Offset_AD[0] +
                 DA_Value[0][pMeasDependencies->Switch[CHANNEL1]]
               );
 AdjustOffsetDA( CHANNEL2,
                 pMeasDependencies->DCLevel[1] +
                 Offset_AD[1] +
                 DA_Value[1][pMeasDependencies->Switch[CHANNEL2]]
               );

/*
 * set the reference voltages for the AD converter
 */
 SetReferenceVoltageAD( Ref_Value[0][pMeasDependencies->Switch[CHANNEL2]],
                        Ref_Value [1][pMeasDependencies->Switch[CHANNEL2]]
                      );
/*
 * set the PGA, with the coupling and switch settings
 */
 SetPGA( pMeasDependencies->ACDC[CHANNEL1],
         pMeasDependencies->ACDC[CHANNEL2],
         SwitchSettings
       );

/*
 * prepare some more settings
 */
 TP508_OverallInfo.SampleFreq      = pMeasDependencies->SampleFreq;
 TP508_OverallInfo.NumPostSamples  = pMeasDependencies->NumPostSamples;
 TP508_OverallInfo.NumPreSamples   = pMeasDependencies->NumPreSamples;
 TP508_OverallInfo.TrigHysteresis  = pMeasDependencies->TrigHysteresis;
 TP508_OverallInfo.TrigLevel       = pMeasDependencies->TrigLevel;
 TP508_OverallInfo.TrigSlope       = pMeasDependencies->TrigSlope;
 TP508_OverallInfo.TrigSource      = pMeasDependencies->TrigSource;
 TP508_OverallInfo.MeasurementMode = pLIVEInfo->MeasurementMode;

/*
 * perform one measurement, with the prepared settings
 */
 MeasureTP508( &TP508_OverallInfo );
 _disable();
 lTimerCount = 0;
 _enable();
 do
 {
  /*
   * increase timer counter until End Of Conversion; if TimeOut value is
   * reached, set the triggerbit
   */
  if (lTimerCount >= pMeasDependencies->TimeOut)
    SetTrigBit();
 }
 while (!ReadEOCBit());

 if (ReadEOCBit())
 {
  /*
   * if conversion is ready, copy waveforms to PC RAM,
   * extract the samples and display them
   */
   RetrieveDataFromBoard (&TP508_OverallInfo);
   ExtractSamples (pLIVEInfo);
   DisplayMeasuredSignals (pLIVEInfo);
 }
}//StartMeasuringTP508



/* NAME:
 *   void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies)
 * DESCRIPTION:
 *   This routine initializes the instrument settings
 * INPUT PARAMETERS:
 *   -
 * OUTPUT PARAMETERS:
 *   pMeasDependencies   this structure will contain the instrument setup
 */
void InitializeMeasurementDependencies (MEASDEP* pMeasDependencies)
{
 pMeasDependencies->Switch[CHANNEL1]   = PGA4VOLT;
 pMeasDependencies->Switch[CHANNEL2]   = PGA4VOLT;
 pMeasDependencies->ACDC[CHANNEL1]     = COUPLEDC;
 pMeasDependencies->ACDC[CHANNEL2]     = COUPLEAC;
 pMeasDependencies->TrigSource         = CH1;
 pMeasDependencies->TrigLevel          = 128;       // 128 = midscreen
 pMeasDependencies->TrigHysteresis     = 1;
 pMeasDependencies->TrigSlope          = NEG;
 pMeasDependencies->NumPreSamples      = 0;
 pMeasDependencies->NumPostSamples     = 400;
 pMeasDependencies->DCLevel[CHANNEL1]  = 0;
 pMeasDependencies->DCLevel[CHANNEL2]  = 0;
 pMeasDependencies->SampleFreq         = 5.5e6;
 pMeasDependencies->TimeOut            = 20;   // 20 * (1/18.2) sec

} //InitializeMeasurementDependencies
