program MeasureAD128;
(*
 * DESCRIPTION
 *   This file contains a sample program for the AD128.
 *   It provides for low level routines for all hardware functions of the
 *   AD128.
 *   It also measures all 8 channels and displays the measured values
 *   in a table on the screen of the PC
 *
 *   This program is made in Borland Pascal 7.0, but should work in other
 *   versions as well.
 *
 *   Refer to your AD128 user manual for technical information
 *
 *)

uses  Crt;

const AD128IoAddress = $288; { <- adapt this value to the address your     }
                             {    AD128 is set to, factory default is $300 }
      MaxSamples     = 1000;
      MaxChannels    = 8;

type  MeasureDataArray = array[1..MaxChannels, 0..MaxSamples] of integer;
      GainType         = record
                           byHardwareGain : byte;
                           rSoftwareGain  : real;
                         end;
      InputRangeArray  = array[1..MaxChannels] of GainType;
      ChSelectArray    = array[1..MaxChannels] of byte;

const Range1_25volt : GainType = (byHardwareGain : 4; rSoftwareGain :  1.25);
      Range2_5volt  : GainType = (byHardwareGain : 3; rSoftwareGain :  2.5 );
      Range5volt    : GainType = (byHardwareGain : 2; rSoftwareGain :  5.0 );
      Range10volt   : GainType = (byHardwareGain : 1; rSoftwareGain : 10.0 );
      Range20volt   : GainType = (byHardwareGain : 0; rSoftwareGain : 20.0 );

      ChSelect      : ChSelectArray = ( 0, 8, 16, 24, 32, 40, 48, 56 );

      Ch1 = 1;
      Ch2 = 2;
      Ch3 = 3;
      Ch4 = 4;
      Ch5 = 5;
      Ch6 = 6;
      Ch7 = 7;
      Ch8 = 8;

var   MeasureData     : MeasureDataArray;
      InputRange      : InputRangeArray;
      wMeasureDelay   : word;
      wMeasureSamples : word;
      wSamplesNr      : word;
      wChNr           : word;


function MeasureOneChannel:integer;
(*
 * DESCRIPTION
 *   This routine measures one channel. The value that is read,
 *   is the value of THE PREVIOUS conversion.
 *   In the same time, a new conversion is started.
 * INPUT
 *   -
 * OUTPUT
 *   return value : the measured value
 *)
var nLoData : integer;
begin
  nLoData           :=   port[AD128IoAddress  ];
  MeasureOneChannel := ( port[AD128IoAddress+1] and 15)*256 + nLoData;
end; { MeasureOneChannel }


procedure SetInputRange( byChannel, byRange : byte );
(*
 * DESCRIPTION
 *   This routine selects the channel and the sensitivity for that channel
 * INPUT
 *   byChannel : The requested channel
 *   byRange   : The requested input sensitivity
 * OUTPUT
 *   -
 *)
begin
  port[AD128IoAddress+2] := byChannel + byRange;
end; { SetInputRange }


procedure MeasureChannels( wTotalNr, wMeasureDelay : word;
                           InputRange : InputRangeArray;
                           Var MeasureData : MeasureDataArray
                         );
(*
 * DESCRIPTION
 *   This routine measures all channels and stores the data
 * INPUT
 *   wTotalNr      : the number of samples to measure
 *   wMeasureDelay : the time to wait before the measurement
 *   InputRange    : array holding sensitivities for all channels
 * OUTPUT
 *   MeasureData   : array holding all measured values for each channel
 *)
var wDelayCounter : word;
    wSampleNr     : word;
begin
 (*
  * start at the beginnig of the array
  *)
  wSampleNr    := 0;
 (*
  * Set the input rang for channel 1
  *)
  SetInputRange( ChSelect[Ch1], InputRange[Ch1].byHardwareGain );
 (*
  * start the delay
  *)
  wDelayCounter :=0;
  repeat
    inc( wDelayCounter );
  until wDelayCounter >= wMeasureDelay;
 (*
  * Now perform A DUMMY measurement at channel 1
  * This is done because the routine MeasureOneChannel gives the result of
  * the previous conversion.
  *)
  MeasureData[Ch1,wSampleNr] := MeasureOneChannel;
 (*
  * Now start measuring all channels
  *)
  repeat
   (*
    * next sample
    *)
    inc(wSampleNr);
   (*
    * Setup for measure Ch2 and measure Ch1
    *)
    SetInputRange(ChSelect[Ch2], InputRange[Ch2].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch1,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch3 and measure Ch2
    *)
    SetInputRange(ChSelect[Ch3], InputRange[Ch3].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch2,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch4 and measure Ch3
    *)
    SetInputRange(ChSelect[Ch4], InputRange[Ch4].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch3,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch5 and measure Ch4
    *)
    SetInputRange(ChSelect[Ch5], InputRange[Ch5].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch4,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch6 and measure Ch5
    *)
    SetInputRange(ChSelect[Ch6], InputRange[Ch6].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch5,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch7 and measure Ch6
    *)
    SetInputRange(ChSelect[Ch7], InputRange[Ch7].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch6,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch8 and measure Ch7
    *)
    SetInputRange(ChSelect[Ch8], InputRange[Ch8].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch7,wSampleNr]:=MeasureOneChannel;
   (*
    * Setup for measure Ch1 and measure Ch8
    *)
    SetInputRange(ChSelect[Ch1], InputRange[Ch1].byHardwareGain);
    wDelayCounter:=0;
    repeat
      inc(wDelayCounter);
    until wDelayCounter>=wMeasureDelay;
    MeasureData[Ch8,wSampleNr]:=MeasureOneChannel;

  until wSampleNr >= wTotalNr;

end; { MeasureChannels }


procedure SetDigOutputs( byValue : byte );
(*
 * DESCRIPTION
 *   This routine sets the digital outputs with the requested value
 * INPUT
 *   byValue:  the value to send to the digital outputs
 * OUTPUT
 *   -
 *)
begin
  port[AD128IOAddress+3] := byValue;
end; { SetDigOutputs }


function ReadDigInputs : byte;
(*
 * DESCRIPTION
 *   This routine reads the digital inputs
 * INPUT
 *   -
 * OUTPUT
 *   returnvalue: the data on the digital inputs
 *)
begin
  ReadDigInputs := port[AD128IOAddress+3];
end; { ReadDigInputs }



(*
 * M A I N   P R O G R A M
 *)

begin
  clrscr;
 (*
  * do some initialization
  *)
  InputRange[Ch1] := range1_25volt;
  InputRange[Ch2] := range2_5volt;
  InputRange[Ch3] := range2_5volt;
  InputRange[Ch4] := range5volt;
  InputRange[Ch5] := range10volt;
  InputRange[Ch6] := range20volt;
  InputRange[Ch7] := range20volt;
  InputRange[Ch8] := range20volt;
 (*
  * Now perform a measurement loop
  *)
  repeat
   (*
    * wait 'some' time.
    *
    * For an adequate timing, e.g. 1 sample per msec, you have to determine
    * a proper value for wMeasureDelay. This value will vary from PC to PC
    * the value 1000 in this software is just for example
    *)
    wMeasureDelay   := 1000;
   (*
    * measure 10 samples
    *)
    wMeasureSamples :=  10;
    MeasureChannels(wMeasureSamples,wMeasureDelay,InputRange,MeasureData);
   (*
    * display all measured values
    *)
    clrscr;
    for wSamplesNr:=1 to wMeasureSamples do
    begin
      for wChNr:=1 to MaxChannels do
        write((MeasureData[wChNr,wMeasureSamples]-2048)/2048*InputRange[wChNr].rSoftwareGain:9:3);
      writeln;
    end
  until keypressed;

end.